import argparse, os
from loguru import logger
from json_file_split.split_libs.jsonl_split import split_by_number_of_buckets
from fsai_shared_funcs.string_helper import string_is_true_or_false


def app():

    parser = argparse.ArgumentParser(
        description="Split json and jsonl files into parts."
    )

    parser.add_argument(
        "-i",
        "--input_file_path",
        help="The path to the input json or jsonl file.",
        default="./tests/data/test.jsonl",
        required=True,
        type=str,
    )

    parser.add_argument(
        "-s",
        "--save_to_dir",
        help="The directory to save the jsonl file.",
        required=True,
        default="/tmp/output/",
        type=str,
    )

    parser.add_argument(
        "-j",
        "--output_file_name",
        help="The name of the output jsonl file.",
        default="output.jsonl",
        required=True,
        type=str,
    )

    parser.add_argument(
        "-sb",
        "--split_by",
        help="The method of splitting files: number_of_buckets, or size_of_buckets.",
        default="number_of_buckets",
        type=str,
    )

    parser.add_argument(
        "-bs",
        "--batch_size",
        help="The size of each batch.",
        default=10,
        required=True,
        type=int,
    )

    parser.add_argument(
        "-scf",
        "--save_count_file",
        help="Should a file with the count of files generated be saved.",
        default=False,
        required=False,
        type=string_is_true_or_false,
    )

    parser.add_argument(
        "-cfp",
        "--count_file_path",
        help="The file to save the count of files generated.",
        default="/tmp/count.txt",
        required=False,
        type=str,
    )

    args = vars(parser.parse_args())

    output_file_path = os.path.join(
        args["save_to_dir"],
        args["output_file_name"].lstrip(
            os.path.sep
        ),  # remove leading forward slash which causes join to think path is relative
    )

    logger.debug("Saving output file to: {}".format(output_file_path))

    # Create the directory structure
    os.makedirs(os.path.dirname(output_file_path), exist_ok=True)

    logger.info("Split By: {}".format(args["split_by"]))
    logger.info("Batch Size: {}".format(args["batch_size"]))
    logger.info("Input File Path: {}".format(args["input_file_path"]))
    logger.info("Output File Path: {}".format(output_file_path))

    # If the output file path is a jsonl file
    if output_file_path.endswith(".jsonl"):

        if args["split_by"] == "number_of_buckets":
            saved_files = split_by_number_of_buckets(
                args["batch_size"], args["input_file_path"], output_file_path
            )

        if args["split_by"] == "size_of_buckets":
            raise Exception("split_by size_of_buckets not implemented.")

        if args["save_count_file"] == True:
            with open(args["count_file_path"], "w") as f:
                cnt = len(saved_files)
                logger.debug(
                    "Save the count file path {} ({})".format(
                        args["count_file_path"], cnt
                    )
                )
                f.write(str(cnt))

        logger.info("Save {} files: {}".format(len(saved_files), saved_files))



import math
from beartype import beartype
from loguru import logger
import glob, subprocess, os


def round_up(n, decimals=0):
    multiplier = 10 ** decimals
    return math.ceil(n * multiplier) / multiplier


def round_down(n, decimals=0):
    multiplier = 10 ** decimals
    return math.floor(n * multiplier) / multiplier


def split_by_size_of_buckets():
    raise Exception("split_by_size_of_buckets not implemented.")


import os
import glob


def empty_directory_tmp(dir_path: str):
    files = glob.glob(os.path.join(dir_path, "*"))
    for f in files:
        try:
            os.remove(f)
        except:
            pass


@beartype
def split_by_number_of_buckets(
    number_of_buckets: int, input_file_path: str, output_file_path: str, start: int = 0
):
    output_dir_path = os.path.dirname(output_file_path)
    empty_directory_tmp(output_dir_path)

    num_lines = int(open(input_file_path).read().count("\n"))

    # number_of_buckets
    lines_per_file = int(round_up(num_lines / number_of_buckets))

    # Execute the split as a subprocess and ait
    p = subprocess.Popen(
        [
            "split",
            "-a",  # suffix-length
            "5",
            "-l",
            str(lines_per_file),
            input_file_path,
            "{}.".format(output_file_path),
        ]
    )
    p.wait()

    if p.returncode != 0:
        exit(p.returncode)

    output_file_dir = os.path.dirname(os.path.abspath(output_file_path))

    # Get all files in the output directory
    files = glob.glob(os.path.join(output_file_dir, "*.jsonl.*"))

    saved = []

    # For each file
    for counter, file in enumerate(files, start=start):

        # Get the full file path
        path = os.path.dirname(os.path.abspath(file))

        new_file_path = "{}/{}.jsonl".format(path, counter)

        # Rename the file
        os.rename(file, new_file_path)

        saved.append(new_file_path)

    return saved



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['json_file_split', 'json_file_split.split_libs']

package_data = \
{'': ['*']}

install_requires = \
['beartype>=0.10.4,<0.11.0',
 'fsai-shared-funcs>=0.0.35,<0.0.36',
 'loguru>=0.6.0,<0.7.0']

entry_points = \
{'console_scripts': ['json-file-split = json_file_split.main:app']}

setup_kwargs = {
    'name': 'json-file-split',
    'version': '0.0.74',
    'description': 'Split a json or jsonl file into equal chunks.',
    'long_description': '# json-file-split\nSplit a json or jsonl file into different chunks.\n\n## Installation \n```shell\npip install json-file-split\n```\n\n## Usage\n```shell\njson-file-split \\\n--input_file_path ./tests/data/test.jsonl \\\n--save_to_dir /tmp/output/ \\\n--output_file_name test.jsonl \\\n--split_by number_of_buckets \\\n--batch_size 10\n```',
    'author': 'Michael Mohamed',
    'author_email': 'michael@foundationstack.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/fsai-dev/fsai-cli-tools',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
